@@ -1,7 +1,8 @@
 BIC.rma <- function(object, ...) {
 
-   if (!inherits(object, "rma"))
-      stop("Argument 'object' must be an object of class \"rma\".")
+   mstyle <- .get.mstyle("crayon" %in% .packages())
+
+   .chkclass(class(object), must="rma")
 
    if (missing(...)) {
 
@@ -34,8 +35,9 @@ BIC.rma <- function(object, ...) {
       ### check that all models were fitted to the same data
 
       yis <- lapply(list(object, ...), function(x) as.vector(x$yi))
+
       if (!all(sapply(yis[-1], function(x) identical(x, yis[[1]]))))
-         warning("Models not all fitted to the same data.")
+         warning(mstyle$warning("Models not all fitted to the same data."), call.=FALSE)
 
    }
 
