@@ -2,7 +2,7 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(x), must="rma.uni", notav="rma.uni.selmodel")
+   .chkclass(class(x), must="rma.uni", notav=c("rma.uni.selmodel", "rma.gen"))
 
    na.act <- getOption("na.action")
 
@@ -64,12 +64,12 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    if (is.function(transf)) {
       if (is.null(targs)) {
          pred  <- sapply(pred, transf)
-         se    <- rep(NA,x$k.f)
+         se    <- rep(NA_real_, x$k.f)
          pi.lb <- sapply(pi.lb, transf)
          pi.ub <- sapply(pi.ub, transf)
       } else {
          pred  <- sapply(pred, transf, targs)
-         se    <- rep(NA,x$k.f)
+         se    <- rep(NA_real_, x$k.f)
          pi.lb <- sapply(pi.lb, transf, targs)
          pi.ub <- sapply(pi.ub, transf, targs)
       }
