@@ -11,7 +11,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(object), must="rma.uni", notav=c("robust.rma", "rma.ls"))
+   .chkclass(class(object), must="rma.uni", notav=c("robust.rma", "rma.ls", "rma.gen"))
 
    if (!missing(parm))
       warning(mstyle$warning("Argument 'parm' (currently) ignored."), call.=FALSE)
@@ -70,8 +70,8 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
          type <- "QP"
       }
    } else {
-      #type <- match.arg(type, c("QP", "GENQ", "PL"))
-      if (!is.element(type, c("QP", "GENQ", "PL")))
+      #type <- match.arg(type, c("QP","GENQ","PL"))
+      if (!is.element(type, c("QP","GENQ","PL","Wald","Wald.log","Wald.sqrt")))
          stop(mstyle$stop("Unknown 'type' specified."))
    }
 
@@ -119,8 +119,8 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
 
       ######################################################################
 
-      tau2.lb <- NA
-      tau2.ub <- NA
+      tau2.lb <- NA_real_
+      tau2.ub <- NA_real_
       ci.null <- FALSE ### logical if CI is a null set
       lb.conv <- FALSE ### logical if search converged for lower bound (LB)
       ub.conv <- FALSE ### logical if search converged for upper bound (UB)
@@ -372,8 +372,6 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
       ### PL method ###
       #################
 
-      ### note: cannot actually use this at the moment
-
       if (type == "PL") {
 
          if (con$tau2.min > x$tau2)
@@ -467,6 +465,53 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type,
 
       ######################################################################
 
+      if (is.element(type, c("Wald","Wald.log","Wald.sqrt"))) {
+         crit    <- qnorm(level/2, lower.tail=FALSE)
+         lb.conv <- TRUE
+         ub.conv <- TRUE
+      }
+
+      ###################
+      ### Wald method ###
+      ###################
+
+      if (type == "Wald") {
+
+         tau2.lb <- x$tau2 - crit * x$se.tau2
+         tau2.ub <- x$tau2 + crit * x$se.tau2
+         tau2.lb <- max(ifelse(is.null(x$control$tau2.min), 0, x$control$tau2.min), tau2.lb)
+
+      }
+
+      #######################
+      ### Wald.log method ###
+      #######################
+
+      if (type == "Wald.log") {
+
+         if (x$tau2 >= 0) {
+            tau2.lb <- exp(log(x$tau2) - crit * x$se.tau2 / x$tau2)
+            tau2.ub <- exp(log(x$tau2) + crit * x$se.tau2 / x$tau2)
+            tau2.ub <- max(x$tau2, tau2.ub) # if tau2 is 0, then CI is 0 to tau2
+         }
+
+      }
+
+      ########################
+      ### Wald.sqrt method ###
+      ########################
+
+      if (type == "Wald.sqrt") {
+
+         if (x$tau2 >= 0) {
+            tau2.lb <- (max(0, sqrt(x$tau2) - crit * x$se.tau2 / (2 * sqrt(x$tau2))))^2
+            tau2.ub <- (sqrt(x$tau2) + crit * x$se.tau2 / (2 * sqrt(x$tau2)))^2
+         }
+
+      }
+
+      ######################################################################
+
       if (!lb.conv)
          warning(mstyle$warning("Error in iterative search for the lower bound."), call.=FALSE)
 
