@@ -3,7 +3,7 @@ annotate=TRUE,                                                header=FALSE,
 xlim, alim, olim, ylim, at, steps=5, level=x$level, refline=0, digits=2L, width,
 xlab,             ilab, ilab.xpos, ilab.pos,
 transf, atransf, targs, rows,
-efac=1, pch=15, psize=1,                        col,
+efac=1, pch, psize,                           col,
 lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    #########################################################################
@@ -37,8 +37,13 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    if (missing(at))
       at <- NULL
 
-   if (missing(ilab))
+   mf <- match.call()
+
+   if (missing(ilab)) {
       ilab <- NULL
+   } else {
+      ilab <- .getx("ilab", mf=mf, data=x$data)
+   }
 
    if (missing(ilab.xpos))
       ilab.xpos <- NULL
@@ -46,8 +51,23 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    if (missing(ilab.pos))
       ilab.pos <- NULL
 
-   if (missing(col))
-      col <- NULL
+   if (missing(col)) {
+      col <- "black"
+   } else {
+      col <- .getx("col", mf=mf, data=x$data)
+   }
+
+   if (missing(pch)) {
+      pch <- 15
+   } else {
+      pch <- .getx("pch", mf=mf, data=x$data)
+   }
+
+   if (missing(psize)) {
+      psize <- 1
+   } else {
+      psize <- .getx("psize", mf=mf, data=x$data)
+   }
 
    if (missing(cex))
       cex <- NULL
@@ -61,7 +81,8 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    level <- .level(level)
 
    ### digits[1] for annotations, digits[2] for x-axis labels
-   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+   ### note: digits can also be a list (e.g., digits=list(2,3L)); trailing 0's on the x-axis labels
+   ### are dropped if the value is an integer
 
    if (length(digits) == 1L)
       digits <- c(digits,digits)
@@ -199,24 +220,21 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    if (length(pch) == 1L)
       pch <- rep(pch, k)
+
    if (length(pch) != k)
       stop(mstyle$stop(paste0("Length of the 'pch' argument (", length(pch), ") does not correspond to the number of outcomes (", k, ").")))
 
    if (length(psize) == 1L)
       psize <- rep(psize, k)
+
    if (length(psize) != k)
       stop(mstyle$stop(paste0("Length of the 'psize' argument (", length(psize), ") does not correspond to the number of outcomes (", k, ").")))
 
-   ### if user has set the col argument
+   if (length(col) == 1L)
+      col <- rep(col, k)
 
-   if (!is.null(col)) {
-      if (length(col) == 1L)
-         col <- rep(col, k)
-      if (length(col) != k)
-         stop(mstyle$stop(paste0("Length of the 'col' argument (", length(col), ") does not correspond to the number of outcomes (", k, ").")))
-   } else {
-      col <- rep("black", k)
-   }
+   if (length(col) != k)
+      stop(mstyle$stop(paste0("Length of the 'col' argument (", length(col), ") does not correspond to the number of outcomes (", k, ").")))
 
    ### set rows value
 
@@ -264,7 +282,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
          rows.new <- rows                       # rearrange rows due to NAs being omitted from plot
          rows.na  <- rows[!not.na]              # shift higher rows down according to number of NAs omitted
-         for (j in seq_len(length(rows.na))) {
+         for (j in seq_along(rows.na)) {
             rows.new[rows >= rows.na[j]] <- rows.new[rows >= rows.na[j]] - 1
          }
          rows <- rows.new[not.na]
@@ -411,12 +429,12 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+         at.lab <- fmtx(sapply(at.lab, atransf), digits[[2]], drop0ifint=TRUE)
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[2]], drop0ifint=TRUE)
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+      at.lab <- fmtx(at.lab, digits[[2]], drop0ifint=TRUE)
    }
 
    #########################################################################
@@ -574,7 +592,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
       }
 
-      annotext <- .fcf(annotext, digits[[1]])
+      annotext <- fmtx(annotext, digits[[1]])
       annotext <- sub("-", annosym[4], annotext, fixed=TRUE)
 
       if (missing(width)) {
@@ -630,7 +648,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### add some additional stuff to be put into .metafor environment, so that it can be used by addpoly()
 
-   sav <- c(res, list(level=level, annotate=annotate, digits=digits[1], width=width, transf=transf, atransf=atransf, targs=targs, fonts=fonts[1:2], annosym=annosym))
+   sav <- c(res, list(level=level, annotate=annotate, digits=digits[[1]], width=width, transf=transf, atransf=atransf, targs=targs, fonts=fonts[1:2], annosym=annosym))
    try(assign("forest", sav, envir=.metafor), silent=TRUE)
 
    invisible(res)
