@@ -70,11 +70,15 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    level <- .level(level)
 
-   ### digits[1] for annotations, digits[2] for x-axis labels
-   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+   ### digits[1] for annotations, digits[2] for x-axis labels, digits[3] (if specified) for weights
+   ### note: digits can also be a list (e.g., digits=list(2,3L)); trailing 0's on the x-axis labels
+   ### are dropped if the value is an integer
 
    if (length(digits) == 1L)
-      digits <- c(digits,digits)
+      digits <- c(digits,digits,digits)
+
+   if (length(digits) == 2L)
+      digits <- c(digits,digits[[1]])
 
    ddd <- list(...)
 
@@ -224,11 +228,9 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    ###       must have the same length as yi (including NAs) even when subsetting eventually
 
    if (missing(slab)) {
-      if (!is.null(attr(yi, "slab")) && length(attr(yi, "slab")) == k) {
-         slab <- attr(yi, "slab")               # use slab info if it can be found in slab attribute of yi (and it has the right length)
-      } else {
+      slab <- attr(yi, "slab")                  # use slab info if it can be found in slab attribute of yi (and it has the right length)
+      if (is.null(slab) || length(slab) != k)
          slab <- paste("Study", seq_len(k))
-      }
    } else {
       if (length(slab) == 1L && is.na(slab))    # slab=NA can be used to suppress study labels
          slab <- rep("", k)
@@ -271,14 +273,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### adjust subset if specified
 
-   if (!is.null(subset)) {
-
-      subset <- .setnafalse(subset, k=k)
-
-      if (length(subset) != k)
-         stop(mstyle$stop(paste0("Length of the 'subset' argument (", length(subset), ") does not correspond to the number of outcomes (", k, ").")))
-
-   }
+   subset <- .chksubset(subset, k)
 
    ### sort the data if requested
 
@@ -322,15 +317,15 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
    ### if a subset of studies is specified
 
    if (!is.null(subset)) {
-      yi    <- yi[subset]
-      vi    <- vi[subset]
-      ci.lb <- ci.lb[subset]
-      ci.ub <- ci.ub[subset]
-      slab  <- slab[subset]
-      ilab  <- ilab[subset,,drop=FALSE]         # if NULL, remains NULL
-      pch   <- pch[subset]
-      psize <- psize[subset]                    # if NULL, remains NULL
-      col   <- col[subset]
+      yi    <- .getsubset(yi,    subset)
+      vi    <- .getsubset(vi,    subset)
+      ci.lb <- .getsubset(ci.lb, subset)
+      ci.ub <- .getsubset(ci.ub, subset)
+      slab  <- .getsubset(slab,  subset)
+      ilab  <- .getsubset(ilab,  subset, col=TRUE) # if NULL, remains NULL
+      pch   <- .getsubset(pch,   subset)
+      psize <- .getsubset(psize, subset)           # if NULL, remains NULL
+      col   <- .getsubset(col,   subset)
    }
 
    k <- length(yi)                              # in case length of k has changed
@@ -381,7 +376,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
          rows.new <- rows                       # rearrange rows due to NAs being omitted from plot
          rows.na  <- rows[!not.na]              # shift higher rows down according to number of NAs omitted
-         for (j in seq_len(length(rows.na))) {
+         for (j in seq_along(rows.na)) {
             rows.new[rows >= rows.na[j]] <- rows.new[rows >= rows.na[j]] - 1
          }
          rows <- rows.new[not.na]
@@ -571,12 +566,12 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+         at.lab <- fmtx(sapply(at.lab, atransf), digits[[2]], drop0ifint=TRUE)
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[2]], drop0ifint=TRUE)
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]]))
+      at.lab <- fmtx(at.lab, digits[[2]], drop0ifint=TRUE)
    }
 
    #########################################################################
@@ -734,10 +729,13 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
       }
 
-      if (showweights)
+      if (showweights) {
          annotext <- cbind(weights, annotext)
+         annotext <- fmtx(annotext, c(digits[[3]], digits[[1]], digits[[1]], digits[[1]]))
+      } else {
+         annotext <- fmtx(annotext, digits[[1]])
+      }
 
-      annotext <- .fcf(annotext, digits[[1]])
       annotext <- sub("-", annosym[4], annotext, fixed=TRUE)
 
       if (missing(width)) {
@@ -797,7 +795,7 @@ lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    ### add some additional stuff to be put into .metafor environment, so that it can be used by addpoly()
 
-   sav <- c(res, list(level=level, annotate=annotate, digits=digits[1], width=width, transf=transf, atransf=atransf, targs=targs, fonts=fonts[1:2], annosym=annosym))
+   sav <- c(res, list(level=level, annotate=annotate, digits=digits[[1]], width=width, transf=transf, atransf=atransf, targs=targs, fonts=fonts[1:2], annosym=annosym))
    try(assign("forest", sav, envir=.metafor), silent=TRUE)
 
    invisible(res)
