@@ -1,4 +1,4 @@
-funnel.default <- function(x, vi, sei, ni, subset, yaxis="sei", xlim, ylim, xlab, ylab,
+funnel.default <- function(x, vi, sei, ni, subset, yaxis="sei", xlim, ylim, xlab, ylab, slab,
 steps=5, at, atransf, targs, digits, level=95,
 back="lightgray", shade="white", hlines="white",
 refline=0, lty=3, pch, col, bg,
@@ -36,10 +36,10 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
       ni <- NULL
 
    if (is.element(yaxis, c("ni", "ninv", "sqrtni", "sqrtninv", "lni"))) {
-      if (is.null(ni) && !is.null(attr(yi, "ni")))
+      if (is.null(ni))
          ni <- attr(yi, "ni")
       if (!is.null(ni) && length(ni) != k)
-         stop(mstyle$stop("Sample size information not of same length as data."))
+      stop(mstyle$stop(paste0("Length of the 'ni' argument (", length(ni), ") does not correspond to the number of outcomes (", k, ").")))
       if (is.null(ni))
          stop(mstyle$stop("No sample size information available."))
    }
@@ -78,12 +78,16 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
    if (!is.null(sei))
       sei[sei < 0] <- 0
 
-   ### get slab from attributes of yi; if not available or it doesn't have the right length, set slab <- 1:k
+   ### if unspecified, get slab from attributes of yi; if not available or it doesn't have the right length, set slab <- 1:k
 
-   slab <- attr(yi, "slab")
+   if (missing(slab)) {
+      slab <- attr(yi, "slab")
+      if (is.null(slab) || length(slab) != k)
+         slab <- seq_along(yi)
+   }
 
-   if (is.null(slab) || length(slab) != k)
-      slab <- seq_along(yi)
+   if (length(slab) != k)
+      stop(mstyle$stop(paste0("Length of the 'slab' argument (", length(slab), ") does not correspond to the number of outcomes (", k, ").")))
 
    ### set y-axis label if not specified
 
@@ -144,7 +148,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
          digits <- c(digits,digits) ### digits[2] for y-axis labels
    }
 
-   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for integers
 
    if (length(lty) == 1L)
       lty <- rep(lty, 2L) ### 1st value = funnel lines, 2nd value = reference line
@@ -186,7 +190,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
    ddd <- list(...)
 
    if (!is.null(ddd$transf))
-      warning("Function does not have a 'transf' argument (use 'atransf' instead).", call.=FALSE)
+      warning("Function does not have a 'transf' argument (use 'atransf' instead).", call.=FALSE, immediate.=TRUE)
 
    lplot     <- function(..., refline2, level2, lty2, transf, ci.res) plot(...)
    labline   <- function(..., refline2, level2, lty2, transf, ci.res) abline(...)
@@ -231,15 +235,15 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
    ### if a subset of studies is specified
 
    if (!is.null(subset)) {
-      subset <- .setnafalse(subset, k=length(yi))
-      yi   <- yi[subset]
-      vi   <- vi[subset]
-      sei  <- sei[subset]
-      ni   <- ni[subset]
-      slab <- slab[subset]
-      pch  <- pch[subset]
-      col  <- col[subset]
-      bg   <- bg[subset]
+      subset <- .chksubset(subset, length(yi))
+      yi   <- .getsubset(yi,   subset)
+      vi   <- .getsubset(vi,   subset)
+      sei  <- .getsubset(sei,  subset)
+      ni   <- .getsubset(ni,   subset)
+      slab <- .getsubset(slab, subset)
+      pch  <- .getsubset(pch,  subset)
+      col  <- .getsubset(col,  subset)
+      bg   <- .getsubset(bg,   subset)
    }
 
    ### check for NAs and act accordingly
@@ -424,7 +428,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
    ### add y-axis
 
-   laxis(side=2, at=seq(from=ylim[1], to=ylim[2], length.out=steps), labels=formatC(seq(from=ylim[1], to=ylim[2], length.out=steps), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]])), ...)
+   laxis(side=2, at=seq(from=ylim[1], to=ylim[2], length.out=steps), labels=fmtx(seq(from=ylim[1], to=ylim[2], length.out=steps), digits[[2]], drop0ifint=TRUE), ...)
 
    ### add horizontal lines
 
@@ -566,12 +570,12 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+         at.lab <- fmtx(sapply(at.lab, atransf), digits[[1]], drop0ifint=TRUE)
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[1]], drop0ifint=TRUE)
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+      at.lab <- fmtx(at.lab, digits[[1]], drop0ifint=TRUE)
    }
 
    ### add x-axis
@@ -644,12 +648,12 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
       ltxt <- sapply(seq_len(lvals), function(i) {
          if (i == 1)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(level[i], lchars), pval2=.fcf(1, lchars)))))
-            #return(as.expression(bquote(p > .(pval), list(pval=.fcf(level[i], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(level[i], lchars), pval2=fmtx(1, lchars)))))
+            #return(as.expression(bquote(p > .(pval), list(pval=fmtx(level[i], lchars)))))
          if (i > 1 && i < lvals)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(level[i], lchars), pval2=.fcf(level[i-1], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(level[i], lchars), pval2=fmtx(level[i-1], lchars)))))
          if (i == lvals)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(0, lchars), pval2=.fcf(level[i-1], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(0, lchars), pval2=fmtx(level[i-1], lchars)))))
       })
 
       pch.l  <- rep(22, lvals)
