@@ -1,4 +1,4 @@
-funnel.rma <- function(x, yaxis="sei", xlim, ylim, xlab, ylab,
+funnel.rma <- function(x, yaxis="sei", xlim, ylim, xlab, ylab, slab,
 steps=5, at, atransf, targs, digits, level=x$level, addtau2=FALSE,
 type="rstandard", back="lightgray", shade="white", hlines="white",
 refline, lty=3, pch, pch.fill, col, bg,
@@ -100,15 +100,24 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
          digits <- c(digits,digits) ### digits[2] for y-axis labels
    }
 
-   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
+   ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for integers
 
    if (length(lty) == 1L)
       lty <- rep(lty, 2L) ### 1st value = funnel lines, 2nd value = reference line
 
-   ### note: pch, col, and bg must be of the same length as the original data passed to rma()
+   ### note: slab, pch, col, and bg must be of the same length as the original data passed to rma()
    ###       so we have to apply the same subsetting (if necessary) and removing of NAs as was
    ###       done during the model fitting (note: NAs are removed further below)
 
+   if (missing(slab)) {
+      slab <- x$slab
+   } else {
+      slab <- .getx("slab", mf=mf, data=x$data)
+      if (length(slab) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'slab' argument (", length(slab), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
+      slab <- .getsubset(slab, x$subset)
+   }
+
    if (length(pch) == 1L) {
       pch.vec <- FALSE
       pch <- rep(pch, x$k.all)
@@ -119,8 +128,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
    if (length(pch) != x$k.all)
       stop(mstyle$stop(paste0("Length of the 'pch' argument (", length(pch), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (!is.null(x$subset))
-      pch <- pch[x$subset]
+   pch <- .getsubset(pch, x$subset)
 
    if (!inherits(x, "rma.uni.trimfill")) {
 
@@ -139,8 +147,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
       if (length(col) != x$k.all)
          stop(mstyle$stop(paste0("Length of the 'col' argument (", length(col), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-      if (!is.null(x$subset))
-         col <- col[x$subset]
+      col <- .getsubset(col, x$subset)
 
       if (missing(bg)) {
          bg <- "white"
@@ -157,8 +164,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
       if (length(bg) != x$k.all)
          stop(mstyle$stop(paste0("Length of the 'bg' argument (", length(bg), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-      if (!is.null(x$subset))
-         bg <- bg[x$subset]
+      bg <- .getsubset(bg, x$subset)
 
    } else {
 
@@ -184,7 +190,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
    ddd <- list(...)
 
    if (!is.null(ddd$transf))
-      warning("Function does not have a 'transf' argument (use 'atransf' instead).", call.=FALSE)
+      warning("Function does not have a 'transf' argument (use 'atransf' instead).", call.=FALSE, immediate.=TRUE)
 
    lplot     <- function(..., refline2, level2, lty2, transf, ci.res) plot(...)
    labline   <- function(..., refline2, level2, lty2, transf, ci.res) abline(...)
@@ -245,7 +251,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
       sei <- sqrt(vi)
       if (!is.null(x$not.na.yivi))
          x$not.na <- x$not.na.yivi
-      slab <- x$slab[x$not.na] ### slab is subsetted but NAs are not removed, so still need to do this here
+      slab <- slab[x$not.na]   ### slab is subsetted but NAs are not removed, so still need to do this here
       pch  <- pch[x$not.na]    ### same for pch
 
       if (!inherits(x, "rma.uni.trimfill")) {
@@ -285,7 +291,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
       sei    <- res$se[not.na]
       ni     <- x$ni.f[not.na]    ### ni can be NULL and can still include NAs
       vi     <- sei^2
-      slab   <- x$slab[not.na]
+      slab   <- slab[not.na]
       pch    <- pch[not.na]
       col    <- col[not.na]
       bg     <- bg[not.na]
@@ -449,7 +455,7 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
    ### add y-axis
 
-   laxis(side=2, at=seq(from=ylim[1], to=ylim[2], length.out=steps), labels=formatC(seq(from=ylim[1], to=ylim[2], length.out=steps), digits=digits[[2]], format="f", drop0trailing=is.integer(digits[[2]])), ...)
+   laxis(side=2, at=seq(from=ylim[1], to=ylim[2], length.out=steps), labels=fmtx(seq(from=ylim[1], to=ylim[2], length.out=steps), digits[[2]], drop0ifint=TRUE), ...)
 
    ### add horizontal lines
 
@@ -610,12 +616,12 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
    if (is.function(atransf)) {
       if (is.null(targs)) {
-         at.lab <- formatC(sapply(at.lab, atransf), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+         at.lab <- fmtx(sapply(at.lab, atransf), digits[[1]], drop0ifint=TRUE)
       } else {
-         at.lab <- formatC(sapply(at.lab, atransf, targs), digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+         at.lab <- fmtx(sapply(at.lab, atransf, targs), digits[[1]], drop0ifint=TRUE)
       }
    } else {
-      at.lab <- formatC(at.lab, digits=digits[[1]], format="f", drop0trailing=is.integer(digits[[1]]))
+      at.lab <- fmtx(at.lab, digits[[1]], drop0ifint=TRUE)
    }
 
    ### add x-axis
@@ -688,12 +694,12 @@ label=FALSE, offset=0.4, legend=FALSE, ...) {
 
       ltxt <- sapply(seq_len(lvals), function(i) {
          if (i == 1)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(level[i], lchars), pval2=.fcf(1, lchars)))))
-            #return(as.expression(bquote(p > .(pval), list(pval=.fcf(level[i], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(level[i], lchars), pval2=fmtx(1, lchars)))))
+            #return(as.expression(bquote(p > .(pval), list(pval=fmtx(level[i], lchars)))))
          if (i > 1 && i < lvals)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(level[i], lchars), pval2=.fcf(level[i-1], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(level[i], lchars), pval2=fmtx(level[i-1], lchars)))))
          if (i == lvals)
-            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=.fcf(0, lchars), pval2=.fcf(level[i-1], lchars)))))
+            return(as.expression(bquote(paste(.(pval1) < p, phantom() <= .(pval2)), list(pval1=fmtx(0, lchars), pval2=fmtx(level[i-1], lchars)))))
       })
 
       pch.l  <- rep(22, lvals)
