@@ -2,6 +2,18 @@
 
 "[.escalc" <- function(x, i, ...) {
 
+   mf <- paste0(deparse1(match.call()), collapse="")
+   has.drop <- grepl("drop = T", mf, fixed=TRUE) || grepl("drop = F", mf, fixed=TRUE)
+
+   if (!missing(i) && nargs()-has.drop > 2L) {
+      mf <- match.call()
+      i <- .getx("i", mf=mf, data=x)
+      # TODO: enable this?
+      # treat missings in a logical vector as FALSE when selecting rows
+      #if (is.logical(i) && length(i) == nrow(x))
+      #   i[is.na(i)] <- FALSE
+   }
+
    dat <- NextMethod("[")
 
    ### add measure attribute back to each yi variable (if that variable is still part of dat)
@@ -11,12 +23,12 @@
 
    for (l in seq_along(yi.names)) {
 
-      #eval(parse(text=paste0("attr(dat$", yi.names[l], ", 'measure') <- attr(x$", yi.names[l], ", 'measure')")))
+      #eval(str2lang(paste0("attr(dat$", yi.names[l], ", 'measure') <- attr(x$", yi.names[l], ", 'measure')")))
       attr(dat[[yi.names[l]]], "measure") <- attr(x[[yi.names[l]]], "measure")
 
       ### if selecting rows, also subset ni and slab attributes and add them back to each yi variable
 
-      if (!missing(i)) {
+      if (!missing(i) && nargs()-has.drop > 2L) {
          attr(dat[[yi.names[l]]], "ni")   <- attr(x[[yi.names[l]]], "ni")[i]
          attr(dat[[yi.names[l]]], "slab") <- attr(x[[yi.names[l]]], "slab")[i]
       }
@@ -25,10 +37,10 @@
 
    ### add var.names and out.names attributes back to object (but only if they exist and only keep variables still in the dataset)
 
-   all.names <- c("yi.names", "vi.names", "sei.names", "zi.names", "ci.lb.names", "ci.ub.names")
+   all.names <- c("yi.names", "vi.names", "sei.names", "zi.names", "pval.names", "ci.lb.names", "ci.ub.names")
 
    for (l in seq_along(all.names)) {
-      if (any(is.element(attr(x, all.names[l]), names(dat)))) ### check if any of the variables still exist in the dataset
+      if (any(is.element(attr(x, all.names[l]), names(dat)))) # check if any of the variables still exist in the dataset
          attr(dat, all.names[l]) <- attr(x, all.names[l])[is.element(attr(x, all.names[l]), names(dat))]
    }
 
@@ -55,6 +67,7 @@ cbind.escalc <- function (..., deparse.level=1) {
    vi.names    <- NULL
    sei.names   <- NULL
    zi.names    <- NULL
+   pval.names  <- NULL
    ci.lb.names <- NULL
    ci.ub.names <- NULL
 
@@ -63,6 +76,7 @@ cbind.escalc <- function (..., deparse.level=1) {
       vi.names    <- c(attr(arg, "vi.names"),    vi.names)
       sei.names   <- c(attr(arg, "sei.names"),   sei.names)
       zi.names    <- c(attr(arg, "zi.names"),    zi.names)
+      pval.names  <- c(attr(arg, "pval.names"),  pval.names)
       ci.lb.names <- c(attr(arg, "ci.lb.names"), ci.lb.names)
       ci.ub.names <- c(attr(arg, "ci.ub.names"), ci.ub.names)
    }
@@ -73,6 +87,7 @@ cbind.escalc <- function (..., deparse.level=1) {
    attr(dat, "vi.names")    <- unique(vi.names)
    attr(dat, "sei.names")   <- unique(sei.names)
    attr(dat, "zi.names")    <- unique(zi.names)
+   attr(dat, "pval.names")  <- unique(pval.names)
    attr(dat, "ci.lb.names") <- unique(ci.lb.names)
    attr(dat, "ci.ub.names") <- unique(ci.ub.names)
 
@@ -105,7 +120,7 @@ rbind.escalc <- function (..., deparse.level=1) {
       yi.pos <- na.omit(yi.pos)[1]
 
       ### just in case
-      if (length(yi.pos) == 0)
+      if (length(yi.pos) == 0L)
          next
 
       ### get 'ni' attribute from all arguments (but only if argument has 'yi' variable)
