@@ -33,7 +33,7 @@
    RSS   <- crossprod(Y,P) %*% Y
 
    if (verbose)
-      cat(mstyle$verbose(paste("tau2 =", formatC(tau2val, digits=digits[["var"]], width=digits[["var"]]+4, format="f"), "  RSS - objective =", formatC(RSS - objective, format="f", digits=digits[["var"]], flag=" "), "\n")))
+      cat(mstyle$verbose(paste("tau2 =", fmtx(tau2val, digits[["var"]], addwidth=4), "  RSS - objective =", fmtx(RSS - objective, digits[["var"]], flag=" "), "\n")))
 
    return(RSS - objective)
 
@@ -64,7 +64,7 @@
    }
 
    if (verbose)
-      cat(mstyle$verbose(paste("tau2 =", formatC(tau2val, digits=digits[["var"]], width=digits[["var"]]+4, format="f"), "  objective =", formatC(res, format="f", digits=digits[["var"]], flag=" "), "\n")))
+      cat(mstyle$verbose(paste("tau2 =", fmtx(tau2val, digits[["var"]], addwidth=4), "  objective =", fmtx(res, digits[["var"]], flag=" "), "\n")))
 
    return(res)
 
@@ -140,7 +140,7 @@
 
    ### show progress
    if (progbar)
-      cat(mstyle$verbose(paste("pval =", formatC(pval, format="f", digits=digits[["pval"]]), " diff =", formatC(diff, format="f", digits=digits[["pval"]], flag=" "), " val =", formatC(val, format="f", digits=digits[["est"]], flag=" "), "\n")))
+      cat(mstyle$verbose(paste("pval =", fmtx(pval, digits[["pval"]]), " diff =", fmtx(diff, digits[["pval"]], flag=" "), " val =", fmtx(val, digits[["est"]], flag=" "), "\n")))
 
    ### penalize negative differences, which should force the CI bound to correspond to a p-value of *at least* level
    diff <- ifelse(diff < 0, diff*10, diff)
@@ -169,14 +169,17 @@
 
 ### -1 times the log likelihood (regular or restricted) for location-scale model
 
-.ll.rma.ls <- function(par, yi, vi, X, Z, reml, k, pX, alpha.val, verbose, digits, REMLf, link, mZ, alpha.min, alpha.max, alpha.transf, tau2.min, tau2.max) {
+.ll.rma.ls <- function(par, yi, vi, X, Z, reml, k, pX, alpha.val, beta.val, verbose, digits, REMLf, link, mZ, alpha.min, alpha.max, alpha.transf, tau2.min, tau2.max, optbeta) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   #beta  <- par[seq_len(pX)]
-   #alpha <- par[-seq_len(pX)]
-
-   alpha <- par
+   if (optbeta) {
+      beta  <- par[seq_len(pX)]
+      beta  <- ifelse(is.na(beta.val), beta, beta.val)
+      alpha <- par[-seq_len(pX)]
+   } else {
+      alpha <- par
+   }
 
    if (alpha.transf)
       alpha <- mapply(.mapfun.alpha, alpha, alpha.min, alpha.max)
@@ -191,53 +194,70 @@
       tau2 <- c(Z %*% alpha)
    }
 
-   if (any(is.na(tau2)) || any(tau2 < tau2.min) || any(tau2 > tau2.max))
-      return(Inf)
-
-   if (any(tau2 < 0)) {
+   if (any(is.na(tau2)) || any(tau2 < tau2.min) || any(tau2 > tau2.max)) {
 
       llval <- -Inf
+      llcomp <- FALSE
 
    } else {
 
-      ### compute weights
-      wi <- 1/(vi + tau2)
+      llcomp <- TRUE
 
-      ### when using this, the optimization only pertains to the parameter(s) in 'alpha', as 'beta' is then fully
-      ### determined by the current value(s) of 'alpha'; this is actually also how the standard RE/ME model is fitted;
-      ### but is this really the best way of doing this? one could also optimize over beta and alpha jointly!
-      W <- diag(wi, nrow=k, ncol=k)
-      stXWX <- try(.invcalc(X=X, W=W, k=k), silent=TRUE)
+      if (any(tau2 < 0)) {
 
-      if (inherits(stXWX, "try-error")) {
-
-         llval <- -Inf
+         llval  <- -Inf
+         llcomp <- FALSE
 
       } else {
 
-         beta <- stXWX %*% crossprod(X,W) %*% as.matrix(yi)
+         ### compute weights / weights matrix
+         wi <- 1/(vi + tau2)
+         W <- diag(wi, nrow=k, ncol=k)
 
-         ### compute residual sum of squares
-         RSS <- sum(wi*c(yi - X %*% beta)^2)
+         if (!optbeta) {
+
+            stXWX <- try(.invcalc(X=X, W=W, k=k), silent=TRUE)
+
+            if (inherits(stXWX, "try-error")) {
+
+               llval  <- -Inf
+               llcomp <- FALSE
+
+            } else {
+
+               beta <- stXWX %*% crossprod(X,W) %*% as.matrix(yi)
+
+            }
 
-         ### log-likelihood (could leave out additive constants)
-         if (!reml) {
-            llval <- -1/2 * (k) * log(2*base::pi) - 1/2 * sum(log(vi + tau2)) - 1/2 * RSS
-         } else {
-            llval <- -1/2 * (k-pX) * log(2*base::pi) + ifelse(REMLf, 1/2 * determinant(crossprod(X), logarithm=TRUE)$modulus, 0) +
-                     -1/2 * sum(log(vi + tau2)) - 1/2 * determinant(crossprod(X,W) %*% X, logarithm=TRUE)$modulus - 1/2 * RSS
          }
 
       }
 
    }
 
+   if (llcomp) {
+
+      ### compute residual sum of squares
+      RSS <- sum(wi*c(yi - X %*% beta)^2)
+
+      ### compute log-likelihood
+      if (!reml) {
+         llval <- -1/2 * (k) * log(2*base::pi) - 1/2 * sum(log(vi + tau2)) - 1/2 * RSS
+      } else {
+         llval <- -1/2 * (k-pX) * log(2*base::pi) + ifelse(REMLf, 1/2 * determinant(crossprod(X), logarithm=TRUE)$modulus, 0) +
+                  -1/2 * sum(log(vi + tau2)) - 1/2 * determinant(crossprod(X,W) %*% X, logarithm=TRUE)$modulus - 1/2 * RSS
+      }
+
+   }
+
    if (!is.null(mZ))
       alpha <- mZ %*% alpha
 
    if (verbose) {
-      cat(mstyle$verbose(paste0("ll = ",          ifelse(is.na(llval), NA, formatC(llval, digits=digits[["fit"]], format="f", flag=" ")), "  ")))
-      cat(mstyle$verbose(paste0("alpha = ", paste(ifelse(is.na(alpha), NA, formatC(alpha, digits=digits[["est"]], format="f", flag=" ")), collapse=" "))))
+      cat(mstyle$verbose(paste0("ll = ",            fmtx(llval, digits[["fit"]], flag=" "), "  ")))
+      if (optbeta)
+         cat(mstyle$verbose(paste0("beta = ", paste(fmtx(beta,  digits[["est"]], flag=" "), collapse=" "), "  ")))
+      cat(mstyle$verbose(paste0("alpha = ",   paste(fmtx(alpha, digits[["est"]], flag=" "), collapse=" "))))
       cat("\n")
    }
 
@@ -245,4 +265,42 @@
 
 }
 
+.rma.ls.ineqfun.pos <- function(par, yi, vi, X, Z, reml, k, pX, alpha.val, beta.val, verbose, digits, REMLf, link, mZ, alpha.min, alpha.max, alpha.transf, tau2.min, tau2.max, optbeta) {
+
+   if (optbeta) {
+      alpha <- par[-seq_len(pX)]
+   } else {
+      alpha <- par
+   }
+
+   if (alpha.transf)
+      alpha <- mapply(.mapfun.alpha, alpha, alpha.min, alpha.max)
+
+   alpha <- ifelse(is.na(alpha.val), alpha, alpha.val)
+
+   tau2 <- c(Z %*% alpha)
+
+   return(tau2)
+
+}
+
+.rma.ls.ineqfun.neg <- function(par, yi, vi, X, Z, reml, k, pX, alpha.val, beta.val, verbose, digits, REMLf, link, mZ, alpha.min, alpha.max, alpha.transf, tau2.min, tau2.max, optbeta) {
+
+   if (optbeta) {
+      alpha <- par[-seq_len(pX)]
+   } else {
+      alpha <- par
+   }
+
+   if (alpha.transf)
+      alpha <- mapply(.mapfun.alpha, alpha, alpha.min, alpha.max)
+
+   alpha <- ifelse(is.na(alpha.val), alpha, alpha.val)
+
+   tau2 <- -c(Z %*% alpha)
+
+   return(tau2)
+
+}
+
 ############################################################################
