@@ -47,7 +47,7 @@ print.rma.glmm <- function(x, digits, showfit=FALSE, signif.stars=getOption("sho
 
    if (showfit) {
       cat("\n")
-      fs <- .fcf(x$fit.stats$ML, digits[["fit"]])
+      fs <- fmtx(x$fit.stats$ML, digits[["fit"]])
       names(fs) <- c("logLik", "deviance", "AIC", "BIC", "AICc")
       cat("\n")
       tmp <- capture.output(print(fs, quote=FALSE, print.gap=2))
@@ -61,44 +61,44 @@ print.rma.glmm <- function(x, digits, showfit=FALSE, signif.stars=getOption("sho
    if (!is.element(x$method, c("FE","EE","CE"))) {
       if (x$int.only) {
          cat(mstyle$text("tau^2 (estimated amount of total heterogeneity): "))
-         cat(mstyle$result(paste0(.fcf(x$tau2, ifelse(abs(x$tau2) <= .Machine$double.eps*10,0,digits[["var"]])), ifelse(is.na(x$se.tau2), "", paste0(" (SE = " , .fcf(x$se.tau2, digits[["sevar"]]), ")")))))
+         cat(mstyle$result(paste0(fmtx(x$tau2, digits[["var"]], thresh=.Machine$double.eps*10), ifelse(is.na(x$se.tau2), "", paste0(" (SE = " , fmtx(x$se.tau2, digits[["sevar"]]), ")")))))
          cat("\n")
          cat(mstyle$text("tau (square root of estimated tau^2 value):      "))
-         cat(mstyle$result(paste0(ifelse(x$tau2>=0, .fcf(sqrt(x$tau2), ifelse(x$tau2 <= .Machine$double.eps*10,0,digits[["var"]])), NA))))
+         cat(mstyle$result(fmtx(.sqrt(x$tau2), digits[["var"]], thresh=.Machine$double.eps*10)))
          cat("\n")
          cat(mstyle$text("I^2 (total heterogeneity / total variability):   "))
-         cat(mstyle$result(paste0(ifelse(is.na(x$I2), NA, .fcf(x$I2, digits[["het"]])), "%")))
+         cat(mstyle$result(paste0(fmtx(x$I2, 2), "%")))
          cat("\n")
          cat(mstyle$text("H^2 (total variability / sampling variability):  "))
-         cat(mstyle$result(paste0(ifelse(is.na(x$H2), NA, .fcf(x$H2, digits[["het"]])))))
+         cat(mstyle$result(fmtx(x$H2, 2)))
       } else {
          cat(mstyle$text("tau^2 (estimated amount of residual heterogeneity):     "))
-         cat(mstyle$result(paste0(.fcf(x$tau2, ifelse(abs(x$tau2) <= .Machine$double.eps*10,0,digits[["var"]])), ifelse(is.na(x$se.tau2), "", paste0(" (SE = " , .fcf(x$se.tau2, digits[["sevar"]]), ")")))))
+         cat(mstyle$result(paste0(fmtx(x$tau2, digits[["var"]], thresh=.Machine$double.eps*10), ifelse(is.na(x$se.tau2), "", paste0(" (SE = " , fmtx(x$se.tau2, digits[["sevar"]]), ")")))))
          cat("\n")
          cat(mstyle$text("tau (square root of estimated tau^2 value):             "))
-         cat(mstyle$result(paste0(ifelse(x$tau2>=0, .fcf(sqrt(x$tau2), ifelse(x$tau2 <= .Machine$double.eps*10,0,digits[["var"]])), NA))))
+         cat(mstyle$result(fmtx(.sqrt(x$tau2), digits[["var"]], thresh=.Machine$double.eps*10)))
          cat("\n")
          cat(mstyle$text("I^2 (residual heterogeneity / unaccounted variability): "))
-         cat(mstyle$result(paste0(ifelse(is.na(x$I2), NA, .fcf(x$I2, digits[["het"]])), "%")))
+         cat(mstyle$result(paste0(fmtx(x$I2, 2), "%")))
          cat("\n")
          cat(mstyle$text("H^2 (unaccounted variability / sampling variability):   "))
-         cat(mstyle$result(paste0(ifelse(is.na(x$H2), NA, .fcf(x$H2, digits[["het"]])))))
+         cat(mstyle$result(fmtx(x$H2, 2)))
       }
       cat("\n\n")
    }
 
    if (!is.na(x$sigma2)) {
       cat(mstyle$text("sigma^2 (estimated amount of study level variability): "))
-      cat(mstyle$result(paste0(.fcf(x$sigma2, ifelse(abs(x$sigma2) <= .Machine$double.eps*10,0,digits[["var"]])))))
+      cat(mstyle$result(fmtx(x$sigma2, digits[["var"]], thresh=.Machine$double.eps*10)))
       cat("\n")
       cat(mstyle$text("sigma (square root of estimated sigma^2 value):        "))
-      cat(mstyle$result(paste0(ifelse(x$sigma2>=0, .fcf(sqrt(x$sigma2), ifelse(x$sigma2 <= .Machine$double.eps*10,0,digits[["var"]])), NA))))
+      cat(mstyle$result(fmtx(.sqrt(x$sigma2), digits[["var"]], thresh=.Machine$double.eps*10)))
       cat("\n\n")
    }
 
    if (!is.na(x$QE.Wld) || !is.na(x$QE.LRT)) {
-      QE.Wld <- .fcf(x$QE.Wld, digits[["test"]])
-      QE.LRT <- .fcf(x$QE.LRT, digits[["test"]])
+      QE.Wld <- fmtx(x$QE.Wld, digits[["test"]])
+      QE.LRT <- fmtx(x$QE.LRT, digits[["test"]])
       nchar.Wld <- nchar(QE.Wld, keepNA=FALSE)
       nchar.LRT <- nchar(QE.LRT, keepNA=FALSE)
 
@@ -113,9 +113,9 @@ print.rma.glmm <- function(x, digits, showfit=FALSE, signif.stars=getOption("sho
          cat(mstyle$section("Tests for Residual Heterogeneity:"))
       }
       cat("\n")
-      cat(mstyle$result(paste0("Wld(df = ", x$QE.df, ") = ", QE.Wld, ", p-val ", .pval(x$QEp.Wld, digits[["pval"]], showeq=TRUE, sep=" "))))
+      cat(mstyle$result(fmtt(x$QE.Wld, "Wld", df=x$QE.df, pval=x$QEp.Wld, digits=digits)))
       cat("\n")
-      cat(mstyle$result(paste0("LRT(df = ", x$QE.df, ") = ", QE.LRT, ", p-val ", .pval(x$QEp.LRT, digits[["pval"]], showeq=TRUE, sep=" "))))
+      cat(mstyle$result(fmtt(x$QE.LRT, "LRT", df=x$QE.df, pval=x$QEp.LRT, digits=digits)))
       cat("\n\n")
    }
 
@@ -123,17 +123,17 @@ print.rma.glmm <- function(x, digits, showfit=FALSE, signif.stars=getOption("sho
       cat(mstyle$section(paste0("Test of Moderators (coefficient", ifelse(x$m == 1, " ", "s "), .format.btt(x$btt),"):")))
       cat("\n")
       if (is.element(x$test, c("knha","adhoc","t"))) {
-         cat(mstyle$result(paste0("F(df1 = ", x$QMdf[1], ", df2 = ", round(x$QMdf[2], 2), ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QM, "F", df1=x$QMdf[1], df2=x$QMdf[2], pval=x$QMp, digits=digits)))
       } else {
-         cat(mstyle$result(paste0("QM(df = ", x$QMdf[1], ") = ", .fcf(x$QM, digits[["test"]]), ", p-val ", .pval(x$QMp, digits[["pval"]], showeq=TRUE, sep=" "))))
+         cat(mstyle$result(fmtt(x$QM, "QM", df=x$QMdf[1], pval=x$QMp, digits=digits)))
       }
       cat("\n\n")
    }
 
    if (is.element(x$test, c("knha","adhoc","t"))) {
-      res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), tval=.fcf(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
+      res.table <- data.frame(estimate=fmtx(c(x$beta), digits[["est"]]), se=fmtx(x$se, digits[["se"]]), tval=fmtx(x$zval, digits[["test"]]), df=round(x$ddf,2), pval=fmtp(x$pval, digits[["pval"]]), ci.lb=fmtx(x$ci.lb, digits[["ci"]]), ci.ub=fmtx(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
    } else {
-      res.table <- data.frame(estimate=.fcf(c(x$beta), digits[["est"]]), se=.fcf(x$se, digits[["se"]]), zval=.fcf(x$zval, digits[["test"]]), pval=.pval(x$pval, digits[["pval"]]), ci.lb=.fcf(x$ci.lb, digits[["ci"]]), ci.ub=.fcf(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
+      res.table <- data.frame(estimate=fmtx(c(x$beta), digits[["est"]]), se=fmtx(x$se, digits[["se"]]), zval=fmtx(x$zval, digits[["test"]]), pval=fmtp(x$pval, digits[["pval"]]), ci.lb=fmtx(x$ci.lb, digits[["ci"]]), ci.ub=fmtx(x$ci.ub, digits[["ci"]]), stringsAsFactors=FALSE)
    }
    rownames(res.table) <- rownames(x$beta)
    signif <- symnum(x$pval, corr=FALSE, na=FALSE, cutpoints=c(0, 0.001, 0.01, 0.05, 0.1, 1), symbols = c("***", "**", "*", ".", " "))
