@@ -6,7 +6,7 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(x), must="rma", notav=c("robust.rma", "rma.ls", "rma.uni.selmodel"))
+   .chkclass(class(x), must="rma", notav=c("robust.rma", "rma.ls", "rma.gen", "rma.uni.selmodel"))
 
    if (missing(transf))
       transf <- FALSE
@@ -42,7 +42,7 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
       }
 
    } else {
-      stop(mstyle$stop("Radial plots only applicable for models without moderators."))
+      stop(mstyle$stop("Radial plots only available for models without moderators."))
    }
 
    if (center) {
@@ -185,9 +185,9 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
    }
 
    len <- ya.xpos
-   xis <- rep(NA_real_,length(atyis))
-   zis <- rep(NA_real_,length(atyis))
-   for (i in seq_len(length(atyis))) {
+   xis <- rep(NA_real_, length(atyis))
+   zis <- rep(NA_real_, length(atyis))
+   for (i in seq_along(atyis)) {
       xis[i] <- sqrt(len^2/(1+(atyis[i]/asp.rat)^2))
       zis[i] <- xis[i]*atyis[i]
    }
@@ -205,11 +205,11 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
 
    len.l <- ya.xpos
    len.u <- ya.xpos + .015*(xlims[2]-xlims[1])
-   xis.l <- rep(NA_real_,length(atyis))
-   zis.l <- rep(NA_real_,length(atyis))
-   xis.u <- rep(NA_real_,length(atyis))
-   zis.u <- rep(NA_real_,length(atyis))
-   for (i in seq_len(length(atyis))) {
+   xis.l <- rep(NA_real_, length(atyis))
+   zis.l <- rep(NA_real_, length(atyis))
+   xis.u <- rep(NA_real_, length(atyis))
+   zis.u <- rep(NA_real_, length(atyis))
+   for (i in seq_along(atyis)) {
       xis.l[i] <- sqrt(len.l^2/(1+(atyis[i]/asp.rat)^2))
       zis.l[i] <- xis.l[i]*atyis[i]
       xis.u[i] <- sqrt(len.u^2/(1+(atyis[i]/asp.rat)^2))
@@ -232,9 +232,9 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
    }
 
    len <- ya.xpos+.02*(xlims[2]-xlims[1])
-   xis <- rep(NA_real_,length(atyis))
-   zis <- rep(NA_real_,length(atyis))
-   for (i in seq_len(length(atyis))) {
+   xis <- rep(NA_real_, length(atyis))
+   zis <- rep(NA_real_, length(atyis))
+   for (i in seq_along(atyis)) {
       xis[i] <- sqrt(len^2/(1+(atyis[i]/asp.rat)^2))
       zis[i] <- xis[i]*atyis[i]
    }
@@ -250,15 +250,15 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
    valid <- zis > zlims[1] & zis < zlims[2]
 
    if (any(valid))
-      text(xis[valid], zis[valid], .fcf(atyis.lab[valid], digits), pos=4, cex=cex, ...)
+      text(xis[valid], zis[valid], fmtx(atyis.lab[valid], digits), pos=4, cex=cex, ...)
 
    ### add CI arc
 
    atyis <- seq(ci.lb, ci.ub, length.out=arc.res[2])
    len <- ci.xpos
-   xis <- rep(NA_real_,length(atyis))
-   zis <- rep(NA_real_,length(atyis))
-   for (i in seq_len(length(atyis))) {
+   xis <- rep(NA_real_, length(atyis))
+   zis <- rep(NA_real_, length(atyis))
+   for (i in seq_along(atyis)) {
       xis[i] <- sqrt(len^2/(1+(atyis[i]/asp.rat)^2))
       zis[i] <- xis[i]*atyis[i]
    }
@@ -273,11 +273,11 @@ back="lightgray", transf, targs, pch=19, arc.res=100, cex, ...) {
    atyis <- c(ci.lb, beta, ci.ub)
    len.l <- ci.xpos-.007*(xlims[2]-xlims[1])
    len.u <- ci.xpos+.007*(xlims[2]-xlims[1])
-   xis.l <- rep(NA_real_,3L)
-   zis.l <- rep(NA_real_,3L)
-   xis.u <- rep(NA_real_,3L)
-   zis.u <- rep(NA_real_,3L)
-   for (i in seq_len(length(atyis))) {
+   xis.l <- rep(NA_real_, 3L)
+   zis.l <- rep(NA_real_, 3L)
+   xis.u <- rep(NA_real_, 3L)
+   zis.u <- rep(NA_real_, 3L)
+   for (i in seq_along(atyis)) {
       xis.l[i] <- sqrt(len.l^2/(1+(atyis[i]/asp.rat)^2))
       zis.l[i] <- xis.l[i]*atyis[i]
       xis.u[i] <- sqrt(len.u^2/(1+(atyis[i]/asp.rat)^2))
