@@ -5,7 +5,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   .chkclass(class(x), must="rma.uni", notav=c("rma.ls", "robust.rma"))
+   .chkclass(class(x), must="rma.uni", notav=c("rma.ls", "rma.gen", "robust.rma"))
 
    alternative <- match.arg(alternative, c("two.sided", "greater", "less"))
 
@@ -26,13 +26,16 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    if (missing(type))
       stop(mstyle$stop("Must choose a specific selection model via the 'type' argument."))
 
-   type.options <- c("beta", "halfnorm", "negexp", "logistic", "power", "negexppow", "halfnorm2", "negexp2", "logistic2", "power2", "stepfun")
+   type.options <- c("beta", "halfnorm", "negexp", "logistic", "power", "negexppow", "halfnorm2", "negexp2", "logistic2", "power2", "stepfun", "trunc", "truncest")
 
    #type <- match.arg(type, type.options)
    type <- type.options[grep(type, type.options)[1]]
    if (is.na(type))
       stop(mstyle$stop("Unknown 'type' specified."))
 
+   if (is.element(type, c("trunc","truncest")) && alternative == "two.sided")
+      stop(mstyle$stop("Cannot use alternative='two-sided' with this type of selection model."))
+
    if (missing(control))
       control <- list()
 
@@ -65,13 +68,14 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
          if (x$tau2.fix) {
             tau2 <- x$tau2
          } else {
-            tau2 <- NA
+            tau2 <- NA_real_
          }
       }
 
    } else {
 
       tau2 <- ddd$tau2
+
       if (!is.na(tau2))
          x$tau2.fix <- TRUE
 
@@ -80,14 +84,14 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    ### handle 'beta' argument from ...
 
    if (is.null(ddd$beta)) {
-      beta <- rep(NA, x$p)
+      beta <- rep(NA_real_, x$p)
       betaspec <- FALSE
    } else {
       beta <- ddd$beta
       betaspec <- TRUE
    }
 
-   yi <- x$yi
+   yi <- c(x$yi)
    vi <- x$vi
    X  <- x$X
    p  <- x$p
@@ -95,7 +99,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ### set precision measure
 
-   if (!missing(prec) && !is.null(prec)) {
+    if (!missing(prec) && !is.null(prec)) {
 
       precspec <- TRUE
 
@@ -122,8 +126,8 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    } else {
 
-      prec <- NULL
       precspec <- FALSE
+      prec <- NULL
       preci <- rep(1, k)
 
    }
@@ -132,14 +136,14 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ### compute p-values
 
-   pvals <- .selmodel.pval(yi=c(yi), vi=vi, alternative=alternative)
+   pvals <- .selmodel.pval(yi=yi, vi=vi, alternative=alternative)
 
    ### checks on steps argument
 
    if (missing(steps) || (length(steps) == 1L && is.na(steps))) {
 
       stepsspec <- FALSE
-      steps <- NA
+      steps <- NA_real_
 
    } else {
 
@@ -148,16 +152,30 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       if (anyNA(steps))
          stop(mstyle$stop("No missing values allowed in 'steps' argument."))
 
-      if (any(steps < 0 | steps > 1))
+      if (type != "trunc" && any(steps < 0 | steps > 1))
          stop(mstyle$stop("Value(s) specified for 'steps' argument must be between 0 and 1."))
 
       steps <- unique(sort(steps))
 
-      if (steps[length(steps)] != 1)
+      if (type != "trunc" && steps[length(steps)] != 1)
          steps <- c(steps, 1)
 
    }
 
+   if (type == "trunc" && !stepsspec) {
+      stepsspec <- TRUE
+      #if (alternative == "greater")
+      #   steps <- min(yi)
+      #if (alternative == "less")
+      #   steps <- max(yi)
+      steps <- 0
+   }
+
+   if (is.element(type, c("trunc","truncest")) && verbose > 2) {
+      warning(mstyle$warning("Cannot use 'verbose > 2' for this type of selection model (setting verbose=2)."), call.=FALSE)
+      verbose <- 2
+   }
+
    ############################################################################
 
    ### set default control parameters
@@ -193,7 +211,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    verbose <- con$verbose
 
-   optimizer <- match.arg(con$optimizer, c("optim","nlminb","uobyqa","newuoa","bobyqa","nloptr","nlm","hjk","nmk","mads","ucminf","lbfgsb3c","subplex","BBoptim","optimParallel","Nelder-Mead","BFGS","CG","L-BFGS-B","SANN","Brent"))
+   optimizer <- match.arg(con$optimizer, c("optim","nlminb","uobyqa","newuoa","bobyqa","nloptr","nlm","hjk","nmk","mads","ucminf","lbfgsb3c","subplex","BBoptim","optimParallel","Nelder-Mead","BFGS","CG","L-BFGS-B","SANN","Brent","Rcgmin","Rvmmin"))
    optmethod <- match.arg(con$optmethod, c("Nelder-Mead","BFGS","CG","L-BFGS-B","SANN","Brent"))
    if (optimizer %in% c("Nelder-Mead","BFGS","CG","L-BFGS-B","SANN","Brent")) {
       optmethod <- optimizer
@@ -268,7 +286,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    if (!x$int.only && x$int.incl && con$scaleX) {
       imX <- try(suppressWarnings(solve(mX)), silent=TRUE)
       if (inherits(imX, "try-error"))
-         stop(mstyle$stop(paste0("Unable to rescale starting values for fixed effects.")))
+         stop(mstyle$stop("Unable to rescale starting values for the fixed effects."))
       beta.init <- c(imX %*% cbind(beta.init))
    }
 
@@ -285,7 +303,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       tau2.init <- log(x$tau2 + 0.001)
    } else {
       if (length(con$tau2.init) != 1L)
-         stop(mstyle$stop(paste0("Argument 'tau2.init' should specify a single value.")))
+         stop(mstyle$stop("Argument 'tau2.init' should specify a single value."))
       if (con$tau2.init <= 0)
          stop(mstyle$stop("Value of 'tau2.init' must be > 0."))
       if (con$tau2.init >= tau2.max)
@@ -293,54 +311,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       tau2.init <- log(con$tau2.init)
    }
 
-   ### set NLOPT_LN_BOBYQA as the default algorithm for nloptr optimizer
-   ### and by default use a relative convergence criterion of 1e-8 on the function value
-
-   if (optimizer == "nloptr" && !is.element("algorithm", names(optcontrol)))
-      optcontrol$algorithm <- "NLOPT_LN_BOBYQA"
-
-   if (optimizer == "nloptr" && !is.element("ftol_rel", names(optcontrol)))
-      optcontrol$ftol_rel <- 1e-8
-
-   ### for mads, set trace=FALSE and tol=1e-6 by default
-
-   if (optimizer == "mads" && !is.element("trace", names(optcontrol)))
-      optcontrol$trace <- FALSE
-
-   if (optimizer == "mads" && !is.element("tol", names(optcontrol)))
-      optcontrol$tol <- 1e-6
-
-   ### for subplex, set reltol=1e-8 by default (the default in subplex() is .Machine$double.eps)
-
-   if (optimizer == "subplex" && !is.element("reltol", names(optcontrol)))
-      optcontrol$reltol <- 1e-8
-
-   ### for BBoptim, set trace=FALSE by default
-
-   if (optimizer == "BBoptim" && !is.element("trace", names(optcontrol)))
-      optcontrol$trace <- FALSE
-
-   ### check that the required packages are installed
-
-   if (is.element(optimizer, c("uobyqa","newuoa","bobyqa"))) {
-      if (!requireNamespace("minqa", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'minqa' package to use this optimizer."))
-   }
-
-   if (is.element(optimizer, c("nloptr","ucminf","lbfgsb3c","subplex","optimParallel"))) {
-      if (!requireNamespace(optimizer, quietly=TRUE))
-         stop(mstyle$stop(paste0("Please install the '", optimizer, "' package to use this optimizer.")))
-   }
-
-   if (is.element(optimizer, c("hjk","nmk","mads"))) {
-      if (!requireNamespace("dfoptim", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'dfoptim' package to use this optimizer."))
-   }
-
-   if (optimizer == "BBoptim") {
-      if (!requireNamespace("BB", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'BB' package to use this optimizer."))
-   }
+   con$hesspack <- match.arg(con$hesspack, c("numDeriv","pracma"))
 
    if (!isTRUE(ddd$skiphes) && !requireNamespace(con$hesspack, quietly=TRUE))
       stop(mstyle$stop(paste0("Please install the '", con$hesspack, "' package to compute the Hessian.")))
@@ -352,6 +323,9 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       if (stepsspec)
          warning(mstyle$warning("Argument 'steps' ignored (not applicable to this type of selection model)."), call.=FALSE)
 
+      stepsspec <- FALSE
+      steps <- NA_real_
+
       if (precspec)
          warning(mstyle$warning("Argument 'prec' ignored (not applicable to this type of selection model)."), call.=FALSE)
 
@@ -493,7 +467,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    if (type == "stepfun") {
 
       if (!stepsspec)
-         stop(mstyle$stop("Must specify the 'steps' argument for this type of selection model."))
+         stop(mstyle$stop("Must specify 'steps' argument for this type of selection model."))
 
       if (precspec)
          warning(mstyle$warning("Adding a precision measure to this selection model is undocumented and experimental."), call.=FALSE)
@@ -509,7 +483,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       delta.min <- rep(0, deltas)
       delta.max <- rep(100, deltas)
       H0.delta <- rep(1, deltas)
-      delta.LRT <- rep(TRUE, deltas) # note: delta[1] should actually not be included, but this gets constrained to 1 below anyway
+      delta.LRT <- rep(TRUE, deltas) # note: delta[1] should actually not be included in the LRT, but gets constrained to 1 below anyway
       pval.min <- 0
       if (type == "stepfun") {
          wi.fun <- function(x, delta, yi, vi, preci, alternative, steps)
@@ -519,12 +493,85 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    }
 
+   if (type == "trunc") {
+
+      if (length(steps) != 1L) # steps should be a single value
+         stop(mstyle$stop("Can only specify a single value for the 'steps' argument for this type of selection model."))
+
+      if (precspec)
+         warning(mstyle$warning("Argument 'prec' ignored (not applicable to this type of selection model)."), call.=FALSE)
+
+      deltas <- 1L
+      delta.transf.fun <- "exp"
+      delta.transf.fun.inv <- "log"
+      delta.lb <- 0
+      delta.ub <- Inf
+      delta.lb.excl <- FALSE
+      delta.ub.excl <- FALSE
+      delta.init <- 1
+      delta.min <- 0
+      delta.max <- 100
+      H0.delta <- 1
+      delta.LRT <- TRUE
+      pval.min <- 0
+      wi.fun <- function(x, delta, yi, vi, preci, alternative, steps) {
+         if (alternative == "less") {
+            yival <- qnorm(x, sd=sqrt(vi), lower.tail=TRUE)
+            ifelse(yival < steps[1], 1, delta)
+         } else {
+            yival <- qnorm(x, sd=sqrt(vi), lower.tail=FALSE)
+            ifelse(yival > steps[1], 1, delta)
+         }
+      }
+      #.selmodel.ll <- ".selmodel.ll.cont"
+      .selmodel.ll <- ".selmodel.ll.trunc"
+
+   }
+
+   if (type == "truncest") {
+
+      if (stepsspec)
+         warning(mstyle$warning("Argument 'steps' ignored (not applicable to this type of selection model)."), call.=FALSE)
+
+      stepsspec <- FALSE
+      steps <- NA_real_
+
+      if (precspec)
+         warning(mstyle$warning("Argument 'prec' ignored (not applicable to this type of selection model)."), call.=FALSE)
+
+      deltas <- 2L
+      delta.transf.fun <- c("exp", "I")
+      delta.transf.fun.inv <- c("log", "I")
+      delta.lb <- c(0, -Inf)
+      delta.ub <- c(Inf, Inf)
+      delta.lb.excl <- c(FALSE, FALSE)
+      delta.ub.excl <- c(FALSE, FALSE)
+      delta.init <- c(1, mean(yi))
+      delta.min <- c(0,   ifelse(alternative=="greater", min(yi)-sd(yi), min(yi)))
+      delta.max <- c(100, ifelse(alternative=="greater", max(yi), max(yi)+sd(yi)))
+      H0.delta <- c(1, 0)
+      delta.LRT <- c(TRUE, FALSE)
+      pval.min <- 0
+      wi.fun <- function(x, delta, yi, vi, preci, alternative, steps) {
+         if (alternative == "less") {
+            yival <- qnorm(x, sd=sqrt(vi), lower.tail=TRUE)
+            ifelse(yival < delta[2], 1, delta[1])
+         } else {
+            yival <- qnorm(x, sd=sqrt(vi), lower.tail=FALSE)
+            ifelse(yival > delta[2], 1, delta[1])
+         }
+      }
+      #.selmodel.ll <- ".selmodel.ll.cont"
+      .selmodel.ll <- ".selmodel.ll.trunc"
+
+   }
+
    ############################################################################
 
    ### checks on delta, delta.init, delta.min, delta.max
 
    if (missing(delta)) {
-      delta <- rep(NA, deltas)
+      delta <- rep(NA_real_, deltas)
    } else {
       if (length(delta) == 1L)
          delta <- rep(delta, deltas)
@@ -636,6 +683,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
          mapfun <- ddd$mapfun
       }
    }
+
    if (is.null(ddd$mapinvfun)) {
       mapinvfun <- rep(NA, deltas)
    } else {
@@ -646,6 +694,11 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       }
    }
 
+   if (type == "truncest") {
+      mapfun[2] <- "I"
+      mapinvfun[2] <- "I"
+   }
+
    delta.init <- mapply(.mapinvfun, delta.init, delta.min, delta.max, mapinvfun)
 
    if (!is.null(con$pval.min))
@@ -659,7 +712,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    pvals[pvals < pval.min]     <- pval.min
    pvals[pvals > (1-pval.min)] <- 1-pval.min
 
-   if (stepsspec) {
+   if (type != "trunc" && stepsspec) {
 
       pgrp     <- sapply(pvals, function(p) which(p <= steps)[1])
       psteps.l <- as.character(c(0,steps[-length(steps)]))
@@ -689,58 +742,18 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ############################################################################
 
-   if (optimizer == "optim") {
-      par.arg <- "par"
-      ctrl.arg <- ", control=optcontrol"
-   }
+   ### model fitting
 
-   if (optimizer == "nlminb") {
-      par.arg <- "start"
-      ctrl.arg <- ", control=optcontrol"
-   }
+   if (verbose > 1)
+      message(mstyle$message("\nModel fitting ...\n"))
 
-   if (is.element(optimizer, c("uobyqa","newuoa","bobyqa"))) {
-      par.arg <- "par"
-      optimizer <- paste0("minqa::", optimizer)
-      ctrl.arg <- ", control=optcontrol"
-   }
+   tmp <- .chkopt(optimizer, optcontrol)
+   optimizer  <- tmp$optimizer
+   optcontrol <- tmp$optcontrol
+   par.arg    <- tmp$par.arg
+   ctrl.arg   <- tmp$ctrl.arg
 
-   if (optimizer == "nloptr") {
-      par.arg <- "x0"
-      optimizer <- paste0("nloptr::nloptr")
-      ctrl.arg <- ", opts=optcontrol"
-   }
-
-   if (optimizer == "nlm") {
-      par.arg <- "p" ### because of this, must use argument name pX for p (number of columns in X matrix)
-      ctrl.arg <- paste(names(optcontrol), unlist(optcontrol), sep="=", collapse=", ")
-      if (nchar(ctrl.arg) != 0L)
-         ctrl.arg <- paste0(", ", ctrl.arg)
-   }
-
-   if (is.element(optimizer, c("hjk","nmk","mads"))) {
-      par.arg <- "par"
-      optimizer <- paste0("dfoptim::", optimizer)
-      ctrl.arg <- ", control=optcontrol"
-   }
-
-   if (is.element(optimizer, c("ucminf","lbfgsb3c","subplex"))) {
-      par.arg <- "par"
-      optimizer <- paste0(optimizer, "::", optimizer)
-      ctrl.arg <- ", control=optcontrol"
-   }
-
-   if (optimizer == "BBoptim") {
-      par.arg <- "par"
-      optimizer <- "BB::BBoptim"
-      ctrl.arg <- ", quiet=TRUE, control=optcontrol"
-   }
-
-   if (optimizer == "optimParallel") {
-
-      par.arg <- "par"
-      optimizer <- paste0("optimParallel::optimParallel")
-      ctrl.arg <- ", control=optcontrol, parallel=parallel"
+   if (optimizer == "optimParallel::optimParallel") {
 
       parallel$cl <- NULL
 
@@ -776,18 +789,13 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    }
 
-   # note: X.fit due to hessian(); pX due to nlm()
-
    optcall <- paste(optimizer, "(", par.arg, "=c(beta.init, tau2.init, delta.init), ",
       .selmodel.ll, ", ", ifelse(optimizer=="optim", "method=optmethod, ", ""),
-      "yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+      "yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
       deltas=deltas, delta.val=delta, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
       tau2.val=tau2, tau2.transf=TRUE, tau2.max=tau2.max, beta.val=beta,
       wi.fun=wi.fun, steps=steps, pgrp=pgrp,
-      alternative=alternative, pval.min=pval.min, intCtrl=intCtrl, verbose=verbose, digits=digits", ctrl.arg, ")\n", sep="")
-
-   if (verbose > 1)
-      message(mstyle$message("\nModel fitting ...\n"))
+      alternative=alternative, pval.min=pval.min, intCtrl=intCtrl, verbose=verbose, digits=digits, dofit=FALSE", ctrl.arg, ")\n", sep="")
 
    #return(optcall)
 
@@ -799,43 +807,9 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    #return(opt.res)
 
-   if (optimizer == "optimParallel::optimParallel" && verbose) {
-      tmp <- capture.output(print(opt.res$loginfo))
-      .print.output(tmp, mstyle$verbose)
-   }
+   ### convergence checks (if verbose print optimParallel log, if verbose > 2 print opt.res, and unify opt.res$par)
 
-   if (inherits(opt.res, "try-error"))
-      stop(mstyle$stop("Error during the optimization. Use verbose=TRUE and see help(selmodel) for more details on the optimization routines."))
-
-   ### convergence checks
-
-   if (is.element(optimizer, c("optim","nlminb","dfoptim::hjk","dfoptim::nmk","optimParallel::optimParallel","lbfgsb3c::lbfgsb3c","subplex::subplex","BB::BBoptim")) && opt.res$convergence != 0)
-      stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (convergence = ", opt.res$convergence, ").")))
-
-   if (is.element(optimizer, c("dfoptim::mads")) && opt.res$convergence > optcontrol$tol)
-      stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (convergence = ", opt.res$convergence, ").")))
-
-   if (is.element(optimizer, c("minqa::uobyqa","minqa::newuoa","minqa::bobyqa")) && opt.res$ierr != 0)
-      stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (ierr = ", opt.res$ierr, ").")))
-
-   if (optimizer=="nloptr::nloptr" && !(opt.res$status >= 1 && opt.res$status <= 4))
-      stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (status = ", opt.res$status, ").")))
-
-   if (optimizer=="ucminf::ucminf" && !(opt.res$convergence == 1 || opt.res$convergence == 2))
-      stop(mstyle$stop(paste0("Optimizer (", optimizer, ") did not achieve convergence (convergence = ", opt.res$convergence, ").")))
-
-   if (verbose > 2) {
-      cat("\n")
-      tmp <- capture.output(print(opt.res))
-      .print.output(tmp, mstyle$verbose)
-   }
-
-   ### copy estimated values to 'par' so code below works
-
-   if (optimizer=="nloptr::nloptr")
-      opt.res$par <- opt.res$solution
-   if (optimizer=="nlm")
-      opt.res$par <- opt.res$estimate
+   opt.res$par <- .chkconv(optimizer=optimizer, opt.res=opt.res, optcontrol=optcontrol, fun="selmodel", verbose=verbose)
 
    ### estimates/values of tau2 and delta on the transformed scale
 
@@ -851,7 +825,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    ### do the final model fit with estimated values
 
    fitcall <- paste(.selmodel.ll, "(par=opt.res$par,
-      yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+      yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
       deltas=deltas, delta.val=delta, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
       tau2.val=tau2, tau2.transf=TRUE, tau2.max=tau2.max, beta.val=beta,
       wi.fun=wi.fun, steps=steps, pgrp=pgrp,
@@ -876,10 +850,10 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ### computing (inverse) Hessian
 
-   H        <- NA
+   H        <- NA_real_
    vb       <- matrix(NA_real_, nrow=p, ncol=p)
-   se.tau2  <- NA
-   vb.delta <- matrix(NA_real_, nrow=deltas, ncol=deltas)
+   se.tau2  <- NA_real_
+   vd       <- matrix(NA_real_, nrow=deltas, ncol=deltas)
 
    if (con$beta.fix) {
       beta.hes <- c(beta)
@@ -913,14 +887,14 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
          if (con$hesspack == "numDeriv")
             hescall <- paste("numDeriv::hessian(", .selmodel.ll, ", x=opt.res$par, method.args=con$hessianCtrl,
-               yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+               yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
                deltas=deltas, delta.val=delta.hes, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
                tau2.val=tau2.hes, tau2.transf=TRUE, tau2.max=tau2.max, beta.val=beta.hes,
                wi.fun=wi.fun, steps=steps, pgrp=pgrp,
                alternative=alternative, pval.min=pval.min, intCtrl=intCtrl, verbose=ifelse(verbose > 3, verbose, 0), digits=digits)\n", sep="")
       if (con$hesspack == "pracma")
             hescall <- paste("pracma::hessian(", .selmodel.ll, ", x0=opt.res$par,
-               yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+               yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
                deltas=deltas, delta.val=delta.hes, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
                tau2.val=tau2.hes, tau2.transf=TRUE, tau2.max=tau2.max, beta.val=beta.hes,
                wi.fun=wi.fun, steps=steps, pgrp=pgrp,
@@ -930,14 +904,14 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
          if (con$hesspack == "numDeriv")
             hescall <- paste("numDeriv::hessian(", .selmodel.ll, ", x=c(beta, tau2, delta), method.args=con$hessianCtrl,
-               yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+               yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
                deltas=deltas, delta.val=delta.hes, delta.transf=FALSE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
                tau2.val=tau2.hes, tau2.transf=FALSE, tau2.max=tau2.max, beta.val=beta.hes,
                wi.fun=wi.fun, steps=steps, pgrp=pgrp,
                alternative=alternative, pval.min=pval.min, intCtrl=intCtrl, verbose=ifelse(verbose > 3, verbose, 0), digits=digits)\n", sep="")
       if (con$hesspack == "pracma")
             hescall <- paste("pracma::hessian(", .selmodel.ll, ", x0=c(beta, tau2, delta),
-               yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+               yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
                deltas=deltas, delta.val=delta.hes, delta.transf=FALSE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
                tau2.val=tau2.hes, tau2.transf=FALSE, tau2.max=tau2.max, beta.val=beta.hes,
                wi.fun=wi.fun, steps=steps, pgrp=pgrp,
@@ -979,7 +953,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
             if (is.na(tau2.hes))
                se.tau2 <- sqrt(iH[c(rep(FALSE,p),TRUE,rep(FALSE,deltas)), c(rep(FALSE,p),TRUE,rep(FALSE,deltas))])
             if (anyNA(delta.hes))
-               vb.delta[is.na(delta.hes), is.na(delta.hes)] <- iH[c(rep(FALSE,p),FALSE,is.na(delta.hes)), c(rep(FALSE,p),FALSE,is.na(delta.hes)), drop=FALSE]
+               vd[is.na(delta.hes), is.na(delta.hes)] <- iH[c(rep(FALSE,p),FALSE,is.na(delta.hes)), c(rep(FALSE,p),FALSE,is.na(delta.hes)), drop=FALSE]
          }
 
       }
@@ -1006,7 +980,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    QM <- try(as.vector(t(beta)[x$btt] %*% chol2inv(chol(vb[x$btt,x$btt])) %*% beta[x$btt]), silent=TRUE)
 
    if (inherits(QM, "try-error"))
-      QM <- NA
+      QM <- NA_real_
 
    QMp <- pchisq(QM, df=x$m, lower.tail=FALSE)
 
@@ -1025,7 +999,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ### inference for delta parameters
 
-   se.delta <- sqrt(diag(vb.delta))
+   se.delta <- sqrt(diag(vd))
 
    if (con$htransf) {
 
@@ -1035,7 +1009,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       ci.ub.delta <- c(delta.transf + crit * se.delta)
       ci.lb.delta <- mapply(.mapfun, ci.lb.delta, delta.min, delta.max, mapfun)
       ci.ub.delta <- mapply(.mapfun, ci.ub.delta, delta.min, delta.max, mapfun)
-      vb.delta <- matrix(NA_real_, nrow=deltas, ncol=deltas)
+      vd <- matrix(NA_real_, nrow=deltas, ncol=deltas)
       se.delta <- rep(NA_real_, deltas)
 
    } else {
@@ -1055,7 +1029,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    if (con$htransf) {
       ci.lb.tau2 <- exp(tau2.transf - crit * se.tau2)
       ci.ub.tau2 <- exp(tau2.transf + crit * se.tau2)
-      se.tau2 <- NA
+      se.tau2 <- NA_real_
    } else {
       ci.lb.tau2 <- tau2 - crit * se.tau2
       ci.ub.tau2 <- tau2 + crit * se.tau2
@@ -1067,8 +1041,8 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
    ### LRT for H0: tau^2 = 0 (only when NOT fitting a FE model)
 
-   LRT.tau2  <- NA
-   LRTp.tau2 <- NA
+   LRT.tau2  <- NA_real_
+   LRTp.tau2 <- NA_real_
 
    if (!x$tau2.fix && !is.element(x$method, c("FE","EE","CE")) && !isTRUE(ddd$skiphet)) {
 
@@ -1080,7 +1054,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
 
       optcall <- paste(optimizer, "(", par.arg, "=c(beta.init, tau2.init, delta.init), ",
          .selmodel.ll, ", ", ifelse(optimizer=="optim", "method=optmethod, ", ""),
-         "yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+         "yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
          deltas=deltas, delta.val=delta.val, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
          tau2.val=0, tau2.transf=FALSE, tau2.max=tau2.max, beta.val=beta.val,
          wi.fun=wi.fun, steps=steps, pgrp=pgrp,
@@ -1094,7 +1068,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
       if (!inherits(opt.res, "try-error")) {
 
          fitcall <- paste(.selmodel.ll, "(par=opt.res$par,
-            yi=yi, vi=vi, X.fit=X, preci=preci, k=k, pX=p, pvals=pvals,
+            yi=yi, vi=vi, X=X, preci=preci, k=k, pX=p, pvals=pvals,
             deltas=deltas, delta.val=delta.val, delta.transf=TRUE, mapfun=mapfun, delta.min=delta.min, delta.max=delta.max,
             tau2.val=0, tau2.transf=FALSE, tau2.max=tau2.max, beta.val=beta.val,
             wi.fun=wi.fun, steps=steps, pgrp=pgrp,
@@ -1140,7 +1114,7 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    BIC.ML  <- -2 * ll.ML +   parms * log(k)
    AICc.ML <- -2 * ll.ML + 2*parms * max(k, parms+2) / (max(k, parms+2) - parms - 1)
 
-   fit.stats <- matrix(c(ll.ML, dev.ML, AIC.ML, BIC.ML, AICc.ML, ll.REML=NA, dev.REML=NA, AIC.REML=NA, BIC.REML=NA, AICc.REML=NA), ncol=2, byrow=FALSE)
+   fit.stats <- matrix(c(ll.ML, dev.ML, AIC.ML, BIC.ML, AICc.ML, ll.REML=NA_real_, dev.REML=NA_real_, AIC.REML=NA_real_, BIC.REML=NA_real_, AICc.REML=NA_real_), ncol=2, byrow=FALSE)
    dimnames(fit.stats) <- list(c("ll","dev","AIC","BIC","AICc"), c("ML","REML"))
    fit.stats <- data.frame(fit.stats)
 
@@ -1168,19 +1142,19 @@ selmodel.rma.uni <- function(x, type, alternative="greater", prec, delta, steps,
    res$ci.lb.tau2 <- ci.lb.tau2
    res$ci.ub.tau2 <- ci.ub.tau2
 
-   res$dfs <- res$ddf <- NA
+   res$dfs <- res$ddf <- NA_integer_
    res$test <- "z"
    res$s2w  <- 1
 
-   res$QE <- res$QEp <- NA
-   res$I2 <- res$H2 <- res$vt <- NA
+   res$QE <- res$QEp <- NA_real_
+   res$I2 <- res$H2 <- res$vt <- NA_real_
    res$R2 <- NULL
 
    res$QM  <- QM
    res$QMp <- QMp
 
    res$delta       <- delta
-   res$vb.delta    <- vb.delta
+   res$vd          <- vd
    res$se.delta    <- se.delta
    res$zval.delta  <- zval.delta
    res$pval.delta  <- pval.delta
