@@ -27,16 +27,19 @@
 
 ############################################################################
 
-transf.rtoz <- function(xi, ...)                   ### resulting value between -Inf (for -1) and +Inf (for +1)
+transf.rtoz <- function(xi) {                 ### resulting value between -Inf (for -1) and +Inf (for +1)
+   xi[xi >  1] <-  1
+   xi[xi < -1] <- -1
    atanh(xi)
+}
 
-transf.ztor <- function(xi, ...)
+transf.ztor <- function(xi)
    tanh(xi)
 
-transf.ztor.int <- function(xi, targs=NULL, ...) {
+transf.ztor.int <- function(xi, targs=NULL) {
 
    if (is.na(xi))
-      return(NA)
+      return(NA_real_)
 
    targs <- .chktargsint(targs)
 
@@ -63,10 +66,10 @@ transf.ztor.int <- function(xi, targs=NULL, ...) {
 
 ############################################################################
 
-transf.exp.int <- function(xi, targs=NULL, ...) {
+transf.exp.int <- function(xi, targs=NULL) {
 
    if (is.na(xi))
-      return(NA)
+      return(NA_real_)
 
    targs <- .chktargsint(targs)
 
@@ -93,16 +96,16 @@ transf.exp.int <- function(xi, targs=NULL, ...) {
 
 ############################################################################
 
-transf.logit <- function(xi, ...)                  ### resulting value between -Inf (for 0) and +Inf (for +1)
+transf.logit <- function(xi)                       ### resulting value between -Inf (for 0) and +Inf (for +1)
    qlogis(xi)
 
-transf.ilogit <- function(xi, ...)
+transf.ilogit <- function(xi)
    plogis(xi)
 
-transf.ilogit.int <- function(xi, targs=NULL, ...) {
+transf.ilogit.int <- function(xi, targs=NULL) {
 
    if (is.na(xi))
-      return(NA)
+      return(NA_real_)
 
    targs <- .chktargsint(targs)
 
@@ -129,20 +132,20 @@ transf.ilogit.int <- function(xi, targs=NULL, ...) {
 
 ############################################################################
 
-transf.arcsin <- function(xi, ...)                 ### resulting value between 0 (for 0) and asin(1) = pi/2 (for 1)
+transf.arcsin <- function(xi)                      ### resulting value between 0 (for 0) and asin(1) = pi/2 (for 1)
    asin(sqrt(xi))
 
-transf.iarcsin <- function(xi, ...) {
+transf.iarcsin <- function(xi) {
    zi <- sin(xi)^2
    zi[xi < 0] <- 0                                 ### if xi value is below 0 (e.g., CI bound), return 0
    zi[xi > asin(1)] <- 1                           ### if xi value is above maximum possible value, return 1
    return(c(zi))
 }
 
-# transf.iarcsin.int <- function(xi, targs=NULL, ...) {
+# transf.iarcsin.int <- function(xi, targs=NULL) {
 #
 #   if (is.na(xi))
-#      return(NA)
+#      return(NA_real_)
 #
 #   targs <- .chktargsint(targs)
 #
@@ -169,13 +172,13 @@ transf.iarcsin <- function(xi, ...) {
 
 ############################################################################
 
-transf.pft <- function(xi, ni, ...) {              ### Freeman-Tukey transformation for proportions
+transf.pft <- function(xi, ni) {                   ### Freeman-Tukey transformation for proportions
    xi <- xi*ni
    zi <- 1/2*(asin(sqrt(xi/(ni+1))) + asin(sqrt((xi+1)/(ni+1))))
    return(c(zi))
 }
 
-transf.ipft <- function(xi, ni, ...) {             ### inverse of Freeman-Tukey transformation for individual proportions
+transf.ipft <- function(xi, ni) {                  ### inverse of Freeman-Tukey transformation for individual proportions
    zi <- suppressWarnings(1/2 * (1 - sign(cos(2*xi)) * sqrt(1 - (sin(2*xi)+(sin(2*xi)-1/sin(2*xi))/ni)^2)))
    zi <- ifelse(is.nan(zi), NA_real_, zi)
    zi[xi > transf.pft(1,ni)] <- 1                  ### if xi is above upper limit, return 1
@@ -183,9 +186,9 @@ transf.ipft <- function(xi, ni, ...) {             ### inverse of Freeman-Tukey
    return(c(zi))
 }
 
-transf.ipft.hm <- function(xi, targs, ...) {       ### inverse of Freeman-Tukey transformation for a collection of proportions
+transf.ipft.hm <- function(xi, targs) {            ### inverse of Freeman-Tukey transformation for a collection of proportions
    if (is.null(targs) || (is.list(targs) && is.null(targs$ni)))
-      stop("Need to specify the sample sizes via the 'targs' argument.", call.=FALSE)
+      stop("Must specify the sample sizes via the 'targs' argument.", call.=FALSE)
    if (is.list(targs)) {
       ni <- targs$ni
    } else {
@@ -201,7 +204,7 @@ transf.ipft.hm <- function(xi, targs, ...) {       ### inverse of Freeman-Tukey
 
 ############################################################################
 
-transf.isqrt <- function(xi, ...) {
+transf.isqrt <- function(xi) {
    zi <- xi*xi
    zi[xi < 0] <- 0                                 ### if xi value is below 0 (e.g., CI bound), return 0
    return(c(zi))
@@ -209,12 +212,12 @@ transf.isqrt <- function(xi, ...) {
 
 ############################################################################
 
-transf.irft <- function(xi, ti, ...) {             ### Freeman-Tukey transformation for incidence rates
+transf.irft <- function(xi, ti) {                  ### Freeman-Tukey transformation for incidence rates
    zi <- 1/2*(sqrt(xi) + sqrt(xi + 1/ti))          ### xi is the incidence rate (not the number of events!)
    return(c(zi))
 }
 
-transf.iirft <- function(xi, ti, ...) {            ### inverse of Freeman-Tukey transformation for incidence rates (see Freeman-Tukey_incidence.r in code directory)
+transf.iirft <- function(xi, ti) {                 ### inverse of Freeman-Tukey transformation for incidence rates (see Freeman-Tukey_incidence.r in code directory)
    #zi <- (1/ti - 2*xi^2 + ti*xi^4)/(4*xi^2*ti)    ### old version where transf.irft was not multiplied by 1/2
    zi <- (1/ti - 8*xi^2 + 16*ti*xi^4)/(16*xi^2*ti) ### xi is the incidence rate (not the number of events!)
    zi <- ifelse(is.nan(zi), NA_real_, zi)
@@ -225,13 +228,13 @@ transf.iirft <- function(xi, ti, ...) {            ### inverse of Freeman-Tukey
 
 ############################################################################
 
-transf.ahw <- function(xi, ...) {                  ### resulting value between 0 (for alpha=0) and 1 (for alpha=1)
+transf.ahw <- function(xi) {                       ### resulting value between 0 (for alpha=0) and 1 (for alpha=1)
    #zi <- (1-xi)^(1/3)
    zi <- 1 - (1-xi)^(1/3)
    return(c(zi))
 }
 
-transf.iahw <- function(xi, ...) {
+transf.iahw <- function(xi) {
    #zi <- 1-xi^3
    zi <- 1 - (1-xi)^3
    zi <- ifelse(is.nan(zi), NA_real_, zi)
@@ -240,16 +243,16 @@ transf.iahw <- function(xi, ...) {
    return(c(zi))
 }
 
-transf.abt <- function(xi, ...) {                  ### Bonett (2002) transformation of alphas (without bias correction)
-#transf.abt <- function(xi, ni, ...) {             ### resulting value between 0 (for alpha=0) to Inf (for alpha=1)
+transf.abt <- function(xi) {                       ### Bonett (2002) transformation of alphas (without bias correction)
+#transf.abt <- function(xi, ni) {                  ### resulting value between 0 (for alpha=0) to Inf (for alpha=1)
    #zi <- log(1-xi) - log(ni/(ni-1))
    #zi <- log(1-xi)
    zi <- -log(1-xi)
    return(c(zi))
 }
 
-transf.iabt <- function(xi, ...) {                 ### inverse of Bonett (2002) transformation
-#transf.iabt <- function(xi, ni, ...) {
+transf.iabt <- function(xi) {                      ### inverse of Bonett (2002) transformation
+#transf.iabt <- function(xi, ni) {
    #zi <- 1 - exp(xi) * ni / (ni-1)
    #zi <- 1 - exp(xi)
    zi <- 1 - exp(-xi)
@@ -260,21 +263,21 @@ transf.iabt <- function(xi, ...) {                 ### inverse of Bonett (2002)
 
 ############################################################################
 
-transf.dtou1 <- function(xi, ...) {
+transf.dtou1 <- function(xi) {
    u2i <- pnorm(abs(xi)/2)
    return((2*u2i - 1) / u2i)
 }
 
-transf.dtou2 <- function(xi, ...)
+transf.dtou2 <- function(xi)
    pnorm(xi/2)
 
-transf.dtou3 <- function(xi, ...)
+transf.dtou3 <- function(xi)
    pnorm(xi)
 
-transf.dtocles <- function(xi, ...)
+transf.dtocles <- function(xi)
    pnorm(xi/sqrt(2))
 
-transf.dtorpb <- function(xi, n1i, n2i, ...) {
+transf.dtorpb <- function(xi, n1i, n2i) {
    if (missing(n1i) || missing(n2i)) {
       hi <- 4
    } else {
@@ -288,14 +291,14 @@ transf.dtorpb <- function(xi, n1i, n2i, ...) {
    return(xi / sqrt(xi^2 + hi))
 }
 
-transf.dtobesd <- function(xi, ...) {
+transf.dtobesd <- function(xi) {
    rpbi <- xi / sqrt(xi^2 + 4)
    return(0.50 + rpbi/2)
 }
 
-transf.dtomd <- function(xi, targs=NULL, ...) {
+transf.dtomd <- function(xi, targs=NULL) {
    if (is.null(targs) || (is.list(targs) && is.null(targs$sd)))
-      stop("Need to specify a standard deviation value via the 'targs' argument.", call.=FALSE)
+      stop("Must specify a standard deviation value via the 'targs' argument.", call.=FALSE)
    if (is.list(targs)) {
       sd <- targs$sd
    } else {
@@ -306,7 +309,7 @@ transf.dtomd <- function(xi, targs=NULL, ...) {
    return(xi * sd)
 }
 
-transf.logortord <- function(xi, pc, ...) {
+transf.lnortord <- function(xi, pc) {
    if (length(pc) == 1L)
       pc <- rep(pc, length(xi))
    if (length(xi) != length(pc))
@@ -316,7 +319,7 @@ transf.logortord <- function(xi, pc, ...) {
    return(exp(xi)*pc / (1 - pc + pc * exp(xi)) - pc)
 }
 
-transf.logortorr <- function(xi, pc, ...) {
+transf.lnortorr <- function(xi, pc) {
    if (length(pc) == 1L)
       pc <- rep(pc, length(xi))
    if (length(xi) != length(pc))
@@ -327,3 +330,23 @@ transf.logortorr <- function(xi, pc, ...) {
 }
 
 ############################################################################
+
+transf.lnortod.norm <- function(xi)
+   xi / 1.65
+
+transf.lnortod.logis <- function(xi)
+   sqrt(3) / base::pi * xi
+
+transf.dtolnor.norm <- function(xi)
+   xi * 1.65
+
+transf.dtolnor.logis <- function(xi)
+   xi / sqrt(3) * base::pi
+
+transf.lnortortet.pearson <- function(xi)
+   cos(base::pi / (1 + sqrt(exp(xi))))
+
+transf.lnortortet.digby <- function(xi)
+   (exp(xi)^(3/4) - 1) / (exp(xi)^(3/4) + 1)
+
+############################################################################
