@@ -1,6 +1,8 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/plots:cumulative_forest_plot
+### see also: https://www.metafor-project.org/doku.php/plots:cumulative_forest_plot
+
+source("settings.r")
 
 context("Checking plots example: cumulative forest plot")
 
@@ -15,9 +17,6 @@ test_that("plot can be drawn for 'rma.uni' object.", {
    ### decrease margins so the full space is used
    par(mar=c(4,4,1,2))
 
-   ### load BCG vaccine data
-   data(dat.bcg, package="metafor")
-
    ### calculate log risk ratios and corresponding sampling variances
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
 
@@ -25,11 +24,11 @@ test_that("plot can be drawn for 'rma.uni' object.", {
    res <- rma(yi, vi, data=dat, slab=paste(author, year, sep=", "))
 
    ### cumulative meta-analysis (in the order of publication year)
-   tmp <- cumul(res, order=order(dat$year))
+   tmp <- cumul(res, order=year)
 
    ### cumulative forest plot
    forest(tmp, xlim=c(-4,2), at=log(c(.125, .25, .5, 1, 2)),
-          atransf=exp, digits=c(2,3), cex=.75, header="Author(s) and Year")
+          atransf=exp, digits=c(2,3), cex=.85, header="Author(s) and Year")
 
    par(opar)
 
@@ -46,18 +45,15 @@ test_that("plot can be drawn for 'rma.mh' object.", {
    ### decrease margins so the full space is used
    par(mar=c(4,4,1,2))
 
-   ### load BCG vaccine data
-   data(dat.bcg, package="metafor")
-
-   ### fit fixed-effects models using the Mantel-Haenszel method
+   ### fit equal-effects models using the Mantel-Haenszel method
    res <- rma.mh(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg, slab=paste(author, year, sep=", "))
 
    ### cumulative meta-analysis (in the order of publication year)
-   tmp <- cumul(res, order=order(dat.bcg$year))
+   tmp <- cumul(res, order=dat.bcg$year)
 
    ### cumulative forest plot
    forest(tmp, xlim=c(-4,2), at=log(c(.125, .25, .5, 1, 2)),
-          atransf=exp, digits=c(2,3), cex=.75, header="Author(s) and Year")
+          atransf=exp, digits=c(2,3), cex=.85, header="Author(s) and Year")
 
    par(opar)
 
@@ -74,19 +70,18 @@ test_that("plot can be drawn for 'rma.peto' object.", {
    ### decrease margins so the full space is used
    par(mar=c(4,4,1,2))
 
-   ### load BCG vaccine data
-   data(dat.bcg, package="metafor")
-
-   ### fit fixed-effects models using Peto's method
+   ### fit equal-effects models using Peto's method
    res <- rma.peto(ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg, slab=paste(author, year, sep=", "))
 
    ### cumulative meta-analysis (in the order of publication year)
-   tmp <- cumul(res, order=order(dat.bcg$year))
+   tmp <- cumul(res, order=dat.bcg$year)
 
    ### cumulative forest plot
    forest(tmp, xlim=c(-4,2), at=log(c(.125, .25, .5, 1, 2)),
-          atransf=exp, digits=c(2,3), cex=.75, header="Author(s) and Year")
+          atransf=exp, digits=c(2,3), cex=.85, header="Author(s) and Year")
 
    par(opar)
 
 })
+
+rm(list=ls())
