@@ -1,10 +1,14 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/plots:normal_qq_plots
+### see also: https://www.metafor-project.org/doku.php/plots:normal_qq_plots
+
+source("settings.r")
 
 context("Checking plots example: normal QQ plots")
 
-test_that("plot can be drawn for rma().", {
+test_that("plot can be drawn for 'rma.uni' object.", {
+
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
 
    skip_on_cran()
 
@@ -13,14 +17,11 @@ test_that("plot can be drawn for rma().", {
    ### set up 2x2 array for plotting
    par(mfrow=c(2,2))
 
-   ### load BCG vaccine data
-   data(dat.bcg, package="metafor")
-
    ### calculate log risk ratios and corresponding sampling variances
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
 
-   ### fit fixed- and random-effects models
-   res1 <- rma(yi, vi, data=dat, method="FE")
+   ### fit equal- and random-effects models
+   res1 <- rma(yi, vi, data=dat, method="EE")
    res2 <- rma(yi, vi, data=dat)
 
    ### fit fixed- and random-effects models with absolute latitude moderator
@@ -28,7 +29,7 @@ test_that("plot can be drawn for rma().", {
    res4 <- rma(yi, vi, mods=~ablat, data=dat)
 
    ### normal QQ plots for the various models
-   qqnorm(res1, main="Fixed-Effects Model")
+   qqnorm(res1, main="Equal-Effects Model")
    qqnorm(res2, main="Random-Effects Model")
    qqnorm(res3, main="Fixed-Effects with Moderators Model")
    qqnorm(res4, main="Mixed-Effects Model")
@@ -40,12 +41,13 @@ test_that("plot can be drawn for rma().", {
 
 })
 
-test_that("plot can be drawn for rma.mh().", {
+test_that("plot can be drawn for 'rma.mh' object.", {
+
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
 
    skip_on_cran()
 
    opar <- par(no.readonly=TRUE)
-   data(dat.bcg, package="metafor")
    res <- rma.mh(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    qqnorm(res)
    qqnorm(res, type="rstudent", label=TRUE)
@@ -53,12 +55,13 @@ test_that("plot can be drawn for rma.mh().", {
 
 })
 
-test_that("plot can be drawn for rma.peto().", {
+test_that("plot can be drawn for 'rma.peto' object.", {
+
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
 
    skip_on_cran()
 
    opar <- par(no.readonly=TRUE)
-   data(dat.bcg, package="metafor")
    res <- rma.peto(ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    qqnorm(res)
    qqnorm(res, type="rstudent", label=TRUE)
@@ -66,11 +69,12 @@ test_that("plot can be drawn for rma.peto().", {
 
 })
 
-test_that("plot cannot be drawn for rma.mv().", {
+test_that("plot cannot be drawn for 'rma.mv' object.", {
 
-   data(dat.bcg, package="metafor")
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
-   res <- rma.mv(yi, vi, random = ~ 1 | trial, data=dat)
+   res <- rma.mv(yi, vi, random = ~ 1 | trial, data=dat, sparse=sparse)
    expect_error(qqnorm(res))
 
 })
+
+rm(list=ls())
